@@ -36,6 +36,7 @@
 #include <KIcon>
 #include <KLineEdit>
 #include <KLocale>
+#include <KWindowSystem>
 
 #include <Plasma/Theme>
 
@@ -88,8 +89,6 @@ QsMatchView::QsMatchView(QWidget *parent)
     d->m_itemsRemoved = false;
     d->m_listVisible = true;
     d->m_selectionMade = false; //Prevent completion box from popping up once a user chooses a match
-    //FIXME: don't hardcode black
-    setStyleSheet("QListWidget {color: black} QLineEdit {color: black}");
 
     d->m_descRect = 0;
     d->m_descText = 0;
@@ -193,8 +192,6 @@ void QsMatchView::setItems(const QList<MatchItem*> &items, bool popup, bool appe
 
     int pos = spacing;
 
-    d->m_currentItem = -1;
-
     if (!append) {
         clear(true);
         d->m_compBox->clear();
@@ -206,6 +203,7 @@ void QsMatchView::setItems(const QList<MatchItem*> &items, bool popup, bool appe
         d->m_items << items;
     }
     qStableSort(d->m_items.begin(), d->m_items.end(), queryMatchCompare);
+    d->m_compBox->clear();
 
     foreach(MatchItem *item, d->m_items) {
         if (item) {
@@ -661,6 +659,18 @@ void QsMatchView::keyPressEvent(QKeyEvent *e)
     QWidget::keyPressEvent(e);
 }
 
+bool QsMatchView::eventFilter(QObject *obj, QEvent *event)
+{
+    if (event->type() == QEvent::MouseButtonPress) {
+        if (KWindowSystem::activeWindow() != winId()) {
+            // this overcomes problems with click-to-focus and being a Dock window
+            KWindowSystem::forceActiveWindow(winId());
+        }
+    }
+
+    return QWidget::eventFilter(obj, event);
+}
+
 } // namespace QuickSand
 
 #include "qs_matchview.moc"
