@@ -61,6 +61,7 @@
 #include "HostConnector.h"
 #include "ProcessController.h"
 #include "ProcessTable.h"
+#include "processui/ksysguardprocesslist.h"
 
 #include "ksysguard.h"
 
@@ -73,12 +74,8 @@ ProcessController *sLocalProcessController = NULL;
 static const char Description[] = I18N_NOOP( "KDE System Monitor" );
 TopLevel* topLevel;
 
-/**
-  This is the constructor for the main widget. It sets up the menu and the
-  TaskMan widget.
- */
 TopLevel::TopLevel()
-  : KXmlGuiWindow( NULL )
+  : KXmlGuiWindow( NULL, Qt::WindowFlags(KDE_DEFAULT_WINDOWFLAGS) | Qt::WindowContextHelpButtonHint)
 {
   QDBusConnection::sessionBus().registerObject("/", this, QDBusConnection::ExportScriptableSlots);
   mTimerId = -1;
@@ -97,6 +94,7 @@ TopLevel::TopLevel()
            SLOT( currentTabChanged( int ) ) );
 
   sLocalProcessController = new ProcessController( this);
+  connect( sLocalProcessController, SIGNAL( processListChanged() ), this, SLOT( updateProcessCount()));
 
   /* Create the status bar. It displays some information about the
    * number of processes and the memory consumption of the local
@@ -118,6 +116,7 @@ TopLevel::TopLevel()
   statusBar()->hide();
 
   // create actions for menu entries
+  mRefreshTabAction = KStandardAction::redisplay(mWorkSpace,SLOT( refreshActiveWorksheet() ),actionCollection());
   mNewWorksheetAction = actionCollection()->addAction("new_worksheet");
   mNewWorksheetAction->setIcon(KIcon("tab-new"));
   connect(mNewWorksheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( newWorkSheet() ));
@@ -145,7 +144,7 @@ TopLevel::TopLevel()
 
   mConfigureSheetAction = actionCollection()->addAction( "configure_sheet" );
   mConfigureSheetAction->setIcon( KIcon("configure") );
-  connect(mConfigureSheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( configure() ));
+  connect(mConfigureSheetAction, SIGNAL(triggered(bool)), SLOT( configureCurrentSheet() ));
 
   retranslateUi();
 }
@@ -153,6 +152,7 @@ TopLevel::TopLevel()
 void TopLevel::retranslateUi()
 {
   setPlainCaption( i18n( "System Monitor" ) );
+  mRefreshTabAction->setText(i18n("&Refresh Tab"));
   mNewWorksheetAction->setText(i18n( "&New Tab..." ));
   mInsertWorksheetAction->setText(i18n( "Import Tab Fr&om File..." ));
   mTabExportAction->setText( i18n( "Save Tab &As..." ) );
@@ -172,7 +172,10 @@ void TopLevel::retranslateUi()
     mQuitAction = KStandardAction::quit( this, SLOT( close() ), actionCollection() );
 }
 
-
+void TopLevel::configureCurrentSheet() {
+  mWorkSpace->configure();
+  mRefreshTabAction->setVisible( mWorkSpace->currentWorkSheet()->updateInterval() == 0 );
+}
 void TopLevel::currentTabChanged(int index)
 {
   QWidget *wdg = mWorkSpace->widget(index);
@@ -184,6 +187,9 @@ void TopLevel::currentTabChanged(int index)
   mHotNewWorksheetUploadAction->setVisible(!locked);
   mMonitorRemoteAction->setVisible(!locked);
 
+  //only show refresh option is update interval is 0 (manual)
+  mRefreshTabAction->setVisible( sheet->updateInterval() == 0 );
+
   if(!locked && !mSensorBrowser) {
     startSensorBrowserWidget();
   }
@@ -334,8 +340,6 @@ void TopLevel::timerEvent( QTimerEvent* )
   if ( statusBar()->isVisibleTo( this ) ) {
     /* Request some info about the memory status. The requested
      * information will be received by answerReceived(). */
-    KSGRD::SensorMgr->sendRequest( "localhost", "pscount",
-                                   (KSGRD::SensorClient*)this, 0 );
     KSGRD::SensorMgr->sendRequest( "localhost", "cpu/idle",
                                    (KSGRD::SensorClient*)this, 1 );
     KSGRD::SensorMgr->sendRequest( "localhost", "mem/physical/free",
@@ -351,7 +355,11 @@ void TopLevel::timerEvent( QTimerEvent* )
   }
 }
 
-void TopLevel::changeEvent( QEvent * event ) 
+void TopLevel::updateProcessCount()  {
+    QString s = i18np( "1 process" "\xc2\x9c" "1", "%1 processes" "\xc2\x9c" "%1", sLocalProcessController->processList()->visibleProcessesCount() );
+    sbProcessCount->setText( s );
+}
+void TopLevel::changeEvent( QEvent * event )
 {
   if (event->type() == QEvent::LanguageChange) {
     KSGRD::SensorMgr->retranslate();
@@ -431,14 +439,8 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
   static qlonglong sFree = 0;
 
   switch ( id ) {
-    case 0:
-      s = i18np( " 1 process ", " %1 processes ", answer.toInt() );
-      sbProcessCount->setText( s );
-
-      break;
-
     case 1:
-      s = i18n( " CPU: %1% ", (int) (100 - answer.toFloat()) );
+      s = i18n( "CPU: %1%\xc2\x9c%1%", (int) (100 - answer.toFloat()) );
       sbCpuStat->setText( s );
       break;
 
@@ -452,7 +454,8 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
 
     case 4:
       mUsedApplication = answer.toLongLong();
-      s = i18nc( "Arguments are formatted byte sizes (used/total)", " Memory: %1 / %2 " ,
+      //Use a multi-length string
+      s = i18nc( "Arguments are formatted byte sizes (used/total)", "Memory: %1 / %2" "\xc2\x9c" "Mem: %1 / %2" "\xc2\x9c" "Mem: %1" "\xc2\x9c" "%1",
                  KGlobal::locale()->formatByteSize( mUsedApplication*1024),
                  KGlobal::locale()->formatByteSize( (mFree+mUsedTotal)*1024 ) );
       sbMemTotal->setText( s );
@@ -481,7 +484,7 @@ void TopLevel::setSwapInfo( qlonglong used, qlonglong free, const QString & )
   if ( used == 0 && free == 0 ) // no swap available
     msg = i18n( " No swap space available " );
   else {
-    msg = i18nc( "Arguments are formatted byte sizes (used/total)", " Swap: %1 / %2 " ,
+    msg = i18nc( "Arguments are formatted byte sizes (used/total)", "Swap: %1 / %2" "\xc2\x9c" "Swap: %1" "\xc2\x9c" "%1" ,
                  KGlobal::locale()->formatByteSize( used*1024 ),
                  KGlobal::locale()->formatByteSize( (free+used)*1024) );
   }
