@@ -365,6 +365,8 @@ void Workspace::takeActivity( Client* c, int flags, bool handled )
         flags &= ~ActivityFocus;
         handled = false; // no point, can't get clicks
         }
+    if( c->clientGroup() && c->clientGroup()->visible() != c )
+        c->clientGroup()->setVisible( c );
     if( !c->isShown( true )) // shouldn't happen, call activateClient() if needed
         {
         kWarning( 1212 ) << "takeActivity: not shown" ;
