@@ -3,6 +3,7 @@
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
+Copyright (C) 2009 Lucas Murray <lmurray@undefinedfire.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -39,6 +40,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <KDE/KPluginFactory>
 #include <KDE/KShortcutsEditor>
 
+#include <KDE/Plasma/FrameSvg>
+#include <KDE/Plasma/Theme>
+
 #include <assert.h>
 #include <limits.h>
 
@@ -57,6 +61,8 @@ class Effect;
 class WindowQuad;
 class GLRenderTarget;
 class GLShader;
+class GLTexture;
+class XRenderPicture;
 class RotationData;
 class WindowQuadList;
 class WindowPrePaintData;
@@ -164,7 +170,7 @@ X-KDE-Library=kwin4_effect_cooleffect
 
 #define KWIN_EFFECT_API_MAKE_VERSION( major, minor ) (( major ) << 8 | ( minor ))
 #define KWIN_EFFECT_API_VERSION_MAJOR 0
-#define KWIN_EFFECT_API_VERSION_MINOR 57
+#define KWIN_EFFECT_API_VERSION_MINOR 112
 #define KWIN_EFFECT_API_VERSION KWIN_EFFECT_API_MAKE_VERSION( \
     KWIN_EFFECT_API_VERSION_MAJOR, KWIN_EFFECT_API_VERSION_MINOR )
 
@@ -176,6 +182,20 @@ enum WindowQuadType
     EFFECT_QUAD_TYPE_START = 100 ///< @internal
     };
 
+/**
+ * EffectWindow::setData() and EffectWindow::data() global roles.
+ * All values between 0 and 999 are reserved for global roles.
+ */
+enum DataRole
+    {
+    // Grab roles are used to force all other animations to ignore the window.
+    // The value of the data is set to the Effect's `this` value.
+    WindowAddedGrabRole = 1,
+    WindowClosedGrabRole,
+    WindowMinimizedGrabRole,
+    WindowUnminimizedGrabRole
+    };
+
 /**
  * Infinite region (i.e. a special region type saying that everything needs to be painted).
  */
@@ -270,7 +290,11 @@ class KWIN_EXPORT Effect
             /**
              * Clear whole background as the very first step, without optimizing it
              **/
-            PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6
+            PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6,
+            /**
+            * Temporary solution since (_OPAQUE | _TRANSLUCENT) is not working currently.
+            **/
+            PAINT_DECORATION_ONLY = 1 << 7
         };
 
         /**
@@ -296,6 +320,11 @@ class KWIN_EXPORT Effect
          */
         virtual void reconfigure( ReconfigureFlags flags );
 
+        /**
+         * Called when another effect requests the proxy for this effect.
+         */
+        virtual void* proxy();
+
         /**
          * Called before starting to paint the screen.
          * In this method you can:
@@ -369,6 +398,8 @@ class KWIN_EXPORT Effect
 
         /** called when moved/resized or once after it's finished */
         virtual void windowUserMovedResized( EffectWindow* c, bool first, bool last );
+        /** called when the geometry changed during moving/resizing. */
+        virtual void windowMoveResizeGeometryUpdate( EffectWindow* c, const QRect& geometry );
         virtual void windowOpacityChanged( EffectWindow* c, double old_opacity );
         virtual void windowAdded( EffectWindow* c );
         virtual void windowClosed( EffectWindow* c );
@@ -376,6 +407,9 @@ class KWIN_EXPORT Effect
         virtual void windowActivated( EffectWindow* c );
         virtual void windowMinimized( EffectWindow* c );
         virtual void windowUnminimized( EffectWindow* c );
+        virtual void clientGroupItemSwitched( EffectWindow* from, EffectWindow* to );
+        virtual void clientGroupItemAdded( EffectWindow* from, EffectWindow* to ); // from merged with to
+        virtual void clientGroupItemRemoved( EffectWindow* c, EffectWindow* group ); // c removed from group
         virtual void windowInputMouseEvent( Window w, QEvent* e );
         virtual void desktopChanged( int old );
         virtual void windowDamaged( EffectWindow* w, const QRect& r );
@@ -396,6 +430,7 @@ class KWIN_EXPORT Effect
         virtual void tabBoxAdded( int mode );
         virtual void tabBoxClosed();
         virtual void tabBoxUpdated();
+        virtual void tabBoxKeyEvent( QKeyEvent* event );
         virtual bool borderActivated( ElectricBorder border );
 
         static int displayWidth();
@@ -462,12 +497,26 @@ class KWIN_EXPORT Effect
  * E.g.  KWIN_EFFECT_CONFIG( flames, MyFlameEffectConfig )
  **/
 #define KWIN_EFFECT_CONFIG( name, classname ) \
-    K_PLUGIN_FACTORY(name##_factory, registerPlugin<classname>();) \
-    K_EXPORT_PLUGIN(name##_factory("kcm_kwineffect_" #name))
-
+    K_PLUGIN_FACTORY(EffectFactory, registerPlugin<classname>(#name);) \
+    K_EXPORT_PLUGIN(EffectFactory("kcm_kwin4_effect_" #name))
+/**
+ * Defines the function used to retrieve multiple effects' config widget
+ * E.g.  KWIN_EFFECT_CONFIG_MULTIPLE( flames,
+ *           KWIN_EFFECT_CONFIG_SINGLE( flames, MyFlameEffectConfig )
+ *           KWIN_EFFECT_CONFIG_SINGLE( fire, MyFireEffectConfig )
+ *           )
+ **/
+#define KWIN_EFFECT_CONFIG_MULTIPLE( name, singles ) \
+    K_PLUGIN_FACTORY(EffectFactory, singles) \
+    K_EXPORT_PLUGIN(EffectFactory("kcm_kwin4_effect_" #name))
+/**
+ * @see KWIN_EFFECT_CONFIG_MULTIPLE
+ */
+#define KWIN_EFFECT_CONFIG_SINGLE( name, classname ) \
+    registerPlugin<classname>(#name);
 /**
  * The declaration of the factory to export the effect
- **/
+ */
 #define KWIN_EFFECT_CONFIG_FACTORY K_PLUGIN_FACTORY_DECLARATION(EffectFactory)
 
 
@@ -508,6 +557,16 @@ class KWIN_EXPORT EffectsHandler
         virtual bool grabKeyboard( Effect* effect ) = 0;
         virtual void ungrabKeyboard() = 0;
 
+        /**
+         * Retrieve the proxy class for an effect if it has one. Will return NULL if
+         * the effect isn't loaded or doesn't have a proxy class.
+         */
+        virtual void* getProxy( QString name ) = 0;
+
+        // Mouse polling
+        virtual void startMousePolling() = 0;
+        virtual void stopMousePolling() = 0;
+
         virtual void checkElectricBorder(const QPoint &pos, Time time) = 0;
         virtual void reserveElectricBorder( ElectricBorder border ) = 0;
         virtual void unreserveElectricBorder( ElectricBorder border ) = 0;
@@ -516,25 +575,100 @@ class KWIN_EXPORT EffectsHandler
         // functions that allow controlling windows/desktop
         virtual void activateWindow( EffectWindow* c ) = 0;
         virtual EffectWindow* activeWindow() const = 0 ;
-        virtual void moveWindow( EffectWindow* w, const QPoint& pos ) = 0;
+        virtual void moveWindow( EffectWindow* w, const QPoint& pos, bool snap = false, double snapAdjust = 1.0 ) = 0;
         virtual void windowToDesktop( EffectWindow* w, int desktop ) = 0;
-        // 
+        virtual void windowToScreen( EffectWindow* w, int screen ) = 0;
+        virtual void setShowingDesktop( bool showing ) = 0;
+
+        // Desktops
+        /**
+         * @returns The ID of the current desktop.
+         */
         virtual int currentDesktop() const = 0;
+        /**
+         * @returns Total number of desktops currently in existence.
+         */
         virtual int numberOfDesktops() const = 0;
+        /**
+         * Set the current desktop to @a desktop.
+         */
         virtual void setCurrentDesktop( int desktop ) = 0;
+        /**
+         * @returns The size of desktop layout in grid units.
+         */
+        virtual QSize desktopGridSize() const = 0;
+        /**
+         * @returns The width of desktop layout in grid units.
+         */
+        virtual int desktopGridWidth() const = 0;
+        /**
+         * @returns The height of desktop layout in grid units.
+         */
+        virtual int desktopGridHeight() const = 0;
+        /**
+         * @returns The width of desktop layout in pixels.
+         */
+        virtual int workspaceWidth() const = 0;
+        /**
+         * @returns The height of desktop layout in pixels.
+         */
+        virtual int workspaceHeight() const = 0;
+        /**
+         * @returns The ID of the desktop at the point @a coords or 0 if no desktop exists at that
+         * point. @a coords is to be in grid units.
+         */
+        virtual int desktopAtCoords( QPoint coords ) const = 0;
+        /**
+         * @returns The coords of desktop @a id in grid units.
+         */
+        virtual QPoint desktopGridCoords( int id ) const = 0;
+        /**
+         * @returns The coords of the top-left corner of desktop @a id in pixels.
+         */
+        virtual QPoint desktopCoords( int id ) const = 0;
+        /**
+         * @returns The ID of the desktop above desktop @a id. Wraps around to the bottom of
+         * the layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        virtual int desktopAbove( int desktop = 0, bool wrap = true ) const = 0;
+        /**
+         * @returns The ID of the desktop to the right of desktop @a id. Wraps around to the
+         * left of the layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        virtual int desktopToRight( int desktop = 0, bool wrap = true ) const = 0;
+        /**
+         * @returns The ID of the desktop below desktop @a id. Wraps around to the top of the
+         * layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        virtual int desktopBelow( int desktop = 0, bool wrap = true ) const = 0;
+        /**
+         * @returns The ID of the desktop to the left of desktop @a id. Wraps around to the
+         * right of the layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        virtual int desktopToLeft( int desktop = 0, bool wrap = true ) const = 0;
+        /**
+         * @returns Whether or not the desktop layout is allowed to be modified by the user.
+         */
+        virtual bool isDesktopLayoutDynamic() const = 0;
+        /**
+         * Create new desktop at the point @a coords
+         * @returns The ID of the created desktop
+         */
+        virtual int addDesktop( QPoint coords ) = 0;
+        /**
+         * Deletes the desktop with the ID @a id. All desktops with an ID greater than the one that
+         * was deleted will have their IDs' decremented.
+         */
+        virtual void deleteDesktop( int id ) = 0;
         virtual QString desktopName( int desktop ) const = 0;
+        virtual bool optionRollOverDesktops() const = 0;
+
         virtual int activeScreen() const = 0; // Xinerama
         virtual int numScreens() const = 0; // Xinerama
         virtual int screenNumber( const QPoint& pos ) const = 0; // Xinerama
         virtual QRect clientArea( clientAreaOption, int screen, int desktop ) const = 0;
         virtual QRect clientArea( clientAreaOption, const EffectWindow* c ) const = 0;
         virtual QRect clientArea( clientAreaOption, const QPoint& p, int desktop ) const = 0;
-        virtual void calcDesktopLayout(int* x, int* y, Qt::Orientation* orientation) const = 0;
-        virtual bool optionRollOverDesktops() const = 0;
-        virtual int desktopToLeft( int desktop, bool wrap ) const = 0;
-        virtual int desktopToRight( int desktop, bool wrap ) const = 0;
-        virtual int desktopUp( int desktop, bool wrap ) const = 0;
-        virtual int desktopDown( int desktop, bool wrap ) const = 0;
         /**
          * Factor by which animation speed in the effect should be modified (multiplied).
          * If configurable in the effect itself, the option should have also 'default'
@@ -593,19 +727,10 @@ class KWIN_EXPORT EffectsHandler
          * Returns @a true if the active window decoration has shadow API hooks.
          */
         virtual bool hasDecorationShadows() const = 0;
-        /**
-         * Returns the textures to be used in the shadow. Textures are mapped
-         * to the quad that has the same list offset. E.g. texture[2] is
-         * rendered where the third QRect that EffectWindow::shadowQuads()
-         * returns is.
-         */
-        virtual QList< QList<QImage> > shadowTextures() = 0;
-        /**
-         * Returns the texture list offset for the requested type.
-         */
-        virtual int shadowTextureList( ShadowType type ) const = 0;
 
         /**
+         * @deprecated
+         * @see EffectFrame
          * Paints given text onto screen, possibly in elided form
          * @param text
          * @param center center point of the painted text
@@ -615,11 +740,23 @@ class KWIN_EXPORT EffectsHandler
          **/
         bool paintText( const QString& text, const QPoint& center, int maxwidth,
                         const QColor& color, const QFont& font = QFont() );
+        /**
+         * @deprecated
+         * @see EffectFrame
+         */
         bool paintText( const QString& text, const QRect& rect, const QColor& color,
                         const QFont& font = QFont(), const Qt::Alignment& alignment = Qt::AlignCenter );
+        /**
+         * @deprecated
+         * @see EffectFrame
+         */
         bool paintTextWithBackground( const QString& text, const QPoint& center, int maxwidth,
                                       const QColor& color, const QColor& bgcolor,
                                       const QFont& font = QFont() );
+        /**
+         * @deprecated
+         * @see EffectFrame
+         */
         bool paintTextWithBackground( const QString& text, const QRect& rect, const QColor& color,
                                       const QColor& bgcolor, const QFont& font = QFont(),
                                       const Qt::Alignment& alignment = Qt::AlignCenter );
@@ -664,13 +801,15 @@ class KWIN_EXPORT EffectWindow
         enum
         {
             /**  Window will not be painted  */
-            PAINT_DISABLED              = 1 << 0,
+            PAINT_DISABLED                 = 1 << 0,
             /**  Window will not be painted because it is deleted  */
-            PAINT_DISABLED_BY_DELETE    = 1 << 1,
+            PAINT_DISABLED_BY_DELETE       = 1 << 1,
             /**  Window will not be painted because of which desktop it's on  */
-            PAINT_DISABLED_BY_DESKTOP   = 1 << 2,
+            PAINT_DISABLED_BY_DESKTOP      = 1 << 2,
             /**  Window will not be painted because it is minimized  */
-            PAINT_DISABLED_BY_MINIMIZE  = 1 << 3
+            PAINT_DISABLED_BY_MINIMIZE     = 1 << 3,
+            /**  Window will not be painted because it is not the active window in a client group  */
+            PAINT_DISABLED_BY_CLIENT_GROUP = 1 << 4
         };
 
         EffectWindow();
@@ -689,6 +828,7 @@ class KWIN_EXPORT EffectWindow
 
         virtual bool isMinimized() const = 0;
         virtual double opacity() const = 0;
+        virtual bool hasAlpha() const = 0;
 
         virtual bool isOnDesktop( int d ) const;
         virtual bool isOnCurrentDesktop() const;
@@ -718,6 +858,7 @@ class KWIN_EXPORT EffectWindow
         virtual QRect contentsRect() const = 0;
         bool hasDecoration() const;
         virtual QByteArray readProperty( long atom, long type, int format ) const = 0;
+        virtual void deleteProperty( long atom ) const = 0;
 
         virtual QString caption() const = 0;
         virtual QPixmap icon() const = 0;
@@ -814,32 +955,35 @@ class KWIN_EXPORT EffectWindow
          * aspects, as opposed to override-redirect windows that are entirely handled by the application).
          */
         virtual bool isManaged() const = 0; // whether it's managed or override-redirect
+        /**
+         * Returns whether or not the window can accept keyboard focus.
+         */
+        virtual bool acceptsFocus() const = 0;
+        /**
+         * Returns whether or not the window is kept above all other windows.
+         */
+        virtual bool keepAbove() const = 0;
 
         virtual bool isModal() const = 0;
         virtual EffectWindow* findModal() = 0;
         virtual EffectWindowList mainWindows() const = 0;
 
-        /**
-         * Returns the positions of the shadow quads to be rendered. All positions
-         * are relative to the window's top-left corner.
-         */
-        virtual QList<QRect> shadowQuads( ShadowType type ) const = 0;
-        /**
-         * Returns the desired opacity of the shadow.
-         */
-        virtual double shadowOpacity( ShadowType type ) const = 0;
-        /**
-         * Returns the desired brightness of the shadow.
-         */
-        virtual double shadowBrightness( ShadowType type ) const = 0;
-        /**
-         * Returns the desired saturation of the shadow.
-         */
-        virtual double shadowSaturation( ShadowType type ) const = 0;
         /**
          * Returns the unmodified window quad list. Can also be used to force rebuilding.
          */
         virtual WindowQuadList buildQuads( bool force = false ) const = 0;
+
+        virtual void minimize() const = 0;
+        virtual void unminimize() const = 0;
+        virtual void closeWindow() const = 0;
+
+        virtual bool visibleInClientGroup() const = 0;
+
+        /**
+         * Can be used to by effects to store arbitrary data in the EffectWindow. 
+         */
+        virtual void setData( int role, const QVariant &data ) = 0;
+        virtual QVariant data( int role ) const = 0;
     };
 
 class KWIN_EXPORT EffectWindowGroup
@@ -1146,7 +1290,7 @@ class KWIN_EXPORT PaintClipper
 class KWIN_EXPORT TimeLine
     {
 
-    Q_ENUMS( CurveShape )
+    //Q_ENUMS( CurveShape ) // Requires Q_OBJECT
 
     public:
         /**
@@ -1169,7 +1313,7 @@ class KWIN_EXPORT TimeLine
          * For small animations, values around 150 milliseconds are sensible.
          * Note that duration 0 is not valid.
          */
-        explicit TimeLine(int duration = 0);
+        explicit TimeLine(const int duration = 0);
 
         /**
          * Creates a copy of the TimeLine so we can have the state copied
@@ -1256,14 +1400,7 @@ class KWIN_EXPORT TimeLine
     };
 
 /**
- * @short A single motion dynamics object.
- *
- * This class represents a single object that can be moved around a
- * n-dimensional space. Although it can be used directly by itself
- * it is recommended to use a motion manager instead.
- * 
- * To create a 1D motion object use Motion<double>
- * To create a 2D motion object use Motion<QRectF>
+ * @internal
  */
 template <typename T>
 class KWIN_EXPORT Motion
@@ -1272,12 +1409,10 @@ class KWIN_EXPORT Motion
         /**
          * Creates a new motion object. "Strength" is the amount of
          * acceleration that is applied to the object when the target
-         * changes and "decay" relates to the amount of overshoot
-         * once the object reaches the target. A decay of 1.0 will
-         * cause never-ending oscillation and while a decay of 0.0
-         * will cause no overshoot.
+         * changes and "smoothness" relates to how fast the object
+         * can change its direction and speed.
          */
-        explicit Motion( T initial = T(), double strength = 7.5, double decay = 0.5 );
+        explicit Motion( T initial, double strength, double smoothness );
         /**
          * Creates an exact copy of another motion object, including
          * position, target and velocity.
@@ -1294,10 +1429,8 @@ class KWIN_EXPORT Motion
 
         inline double strength() const { return m_strength; }
         inline void setStrength( const double strength ) { m_strength = strength; }
-        inline double decay() const { return m_decay; }
-        inline void setDecay( const double decay ) { m_decay = decay; }
-        inline void setStrengthDecay( const double strength, const double decay )
-            { m_strength = strength; m_decay = decay; }
+        inline double smoothness() const { return m_smoothness; }
+        inline void setSmoothness( const double smoothness ) { m_smoothness = smoothness; }
 
         /**
          * The distance between the current position and the target.
@@ -1321,7 +1454,37 @@ class KWIN_EXPORT Motion
         T m_target;
         T m_velocity;
         double m_strength;
-        double m_decay;
+        double m_smoothness;
+    };
+
+/**
+ * @short A single 1D motion dynamics object.
+ *
+ * This class represents a single object that can be moved around a
+ * 1D space. Although it can be used directly by itself it is
+ * recommended to use a motion manager instead.
+ */
+class KWIN_EXPORT Motion1D : public Motion<double>
+    {
+    public:
+        explicit Motion1D( double initial = 0.0, double strength = 0.08, double smoothness = 4.0 );
+        Motion1D( const Motion1D &other );
+        ~Motion1D();
+    };
+
+/**
+ * @short A single 2D motion dynamics object.
+ *
+ * This class represents a single object that can be moved around a
+ * 2D space. Although it can be used directly by itself it is
+ * recommended to use a motion manager instead.
+ */
+class KWIN_EXPORT Motion2D : public Motion<QPointF>
+    {
+    public:
+        explicit Motion2D( QPointF initial = QPointF(), double strength = 0.08, double smoothness = 4.0 );
+        Motion2D( const Motion2D &other );
+        ~Motion2D();
     };
 
 /**
@@ -1363,7 +1526,7 @@ class KWIN_EXPORT WindowMotionManager
          */
         void unmanage( EffectWindow *w );
         /**
-         * Deregister all windows, returning the manager to it's
+         * Deregister all windows, returning the manager to its
          * originally initiated state.
          */
         void unmanageAll();
@@ -1376,7 +1539,7 @@ class KWIN_EXPORT WindowMotionManager
         void calculate( int time );
         /**
          * Modify a registered window's paint data to make it appear
-         * at it's real location on the screen. Usually called in
+         * at its real location on the screen. Usually called in
          * paintWindow(). Remember to flag the window as having been
          * transformed in prePaintWindow() by calling
          * WindowPrePaintData::setTransformed()
@@ -1395,7 +1558,7 @@ class KWIN_EXPORT WindowMotionManager
         void reset( EffectWindow *w );
 
         /**
-         * As the manager to move the window to the target position
+         * Ask the manager to move the window to the target position
          * with the specified scale. If `yScale` is not provided or
          * set to 0.0, `scale` will be used as the scale in the
          * vertical direction as well as in the horizontal direction.
@@ -1420,7 +1583,12 @@ class KWIN_EXPORT WindowMotionManager
          */
         QRectF transformedGeometry( EffectWindow *w ) const;
         /**
-         * Return the window that has it's transformed geometry under
+         * Retrieve the current target geometry of a registered
+         * window.
+         */
+        QRectF targetGeometry( EffectWindow *w ) const;
+        /**
+         * Return the window that has its transformed geometry under
          * the specified point. It is recommended to use the stacking
          * order as it's what the user sees, but it is slightly
          * slower to process.
@@ -1446,17 +1614,131 @@ class KWIN_EXPORT WindowMotionManager
          * or not. Can be used to see if an effect should be
          * processed and displayed or not.
          */
-        inline bool areWindowsMoving() { return m_movingWindows > 0; }
+        inline bool areWindowsMoving() { return !m_movingWindowsSet.isEmpty(); }
 
     private:
         bool m_useGlobalAnimationModifier;
         struct WindowMotion
             { // TODO: Rotation, etc?
-            Motion<QPointF> translation; // Absolute position
-            Motion<QPointF> scale; // xScale and yScale
+            Motion2D translation; // Absolute position
+            Motion2D scale; // xScale and yScale
             };
         QHash<EffectWindow*, WindowMotion> m_managedWindows;
-        uint m_movingWindows;
+        QSet<EffectWindow*> m_movingWindowsSet;
+    };
+
+/**
+ * @short Helper class for displaying text and icons in frames.
+ *
+ * Paints text and/or and icon with an optional frame around them. The
+ * available frames includes one that follows the default Plasma theme and
+ * another that doesn't.
+ * It is recommended to use this class whenever displaying text.
+ */
+class KWIN_EXPORT EffectFrame : public QObject
+    {
+    Q_OBJECT
+
+    public:
+        enum Style
+            {
+            None, ///< Displays no frame around the contents.
+            Unstyled, ///< Displays a basic box around the contents.
+            Styled ///< Displays a Plasma-styled frame around the contents.
+            };
+
+        /**
+         * Creates a new frame object. If the frame does not have a static size
+         * then it will be located at @a position with @a alignment. A
+         * non-static frame will automatically adjust its size to fit the
+         * contents.
+         */
+        explicit EffectFrame( Style style, bool staticSize = true, QPoint position = QPoint( -1, -1 ),
+            Qt::Alignment alignment = Qt::AlignCenter );
+        ~EffectFrame();
+
+        /**
+         * Delete any existing textures to free up graphics memory. They will
+         * be automatically recreated the next time they are required.
+         */
+        void free();
+
+        /**
+         * Render the frame.
+         */
+        void render( QRegion region = infiniteRegion(), double opacity = 1.0, double frameOpacity = 1.0 );
+
+        void setPosition( const QPoint& point );
+        /**
+         * Set the text alignment for static frames and the position alignment
+         * for non-static.
+         */
+        inline void setAlignment( Qt::Alignment alignment )
+            { m_alignment = alignment; }; // Doesn't change geometry
+        void setGeometry( const QRect& geometry, bool force = false );
+        inline QRect geometry() const // Inner/contents geometry
+            { return m_geometry; };
+
+        void setText( const QString& text );
+        inline QString text() const
+            { return m_text; };
+        void setFont( const QFont& font );
+        inline QFont font() const
+            { return m_font; };
+        /**
+         * Set the icon that will appear on the left-hand size of the frame.
+         */
+        void setIcon( const QPixmap& icon );
+        inline QPixmap icon() const
+            { return m_icon; };
+        void setIconSize( const QSize& size );
+        inline QSize iconSize() const
+            { return m_iconSize; };
+
+        /**
+         * The foreground text color as specified by the default Plasma theme.
+         */
+        static QColor styledTextColor();
+
+        /**
+         * Clean up all static texture data. Called when compositing is being disabled.
+         * @internal
+         */
+        static void cleanup();
+
+    private Q_SLOTS:
+        void plasmaThemeChanged();
+
+    private:
+        Q_DISABLE_COPY( EffectFrame ) // As we need to use Qt slots we cannot copy this class
+
+        void autoResize(); // Auto-resize if not a static size
+        void updateTexture(); // Update OpenGL styled frame texture
+        void updateTextTexture(); // Update OpenGL text texture
+        void updatePicture(); // Update XRender styled frame picture
+        void updateTextPicture(); // Update XRender text picture
+
+        Style m_style;
+        Plasma::FrameSvg m_frame;
+        GLTexture* m_texture;
+        GLTexture* m_textTexture;
+        XRenderPicture* m_picture;
+        XRenderPicture* m_textPicture;
+
+        // Position
+        bool m_static;
+        QPoint m_point;
+        Qt::Alignment m_alignment;
+        QRect m_geometry;
+
+        // Contents
+        QString m_text;
+        QFont m_font;
+        QPixmap m_icon;
+        QSize m_iconSize;
+
+        static GLTexture* m_unstyledTexture;
+        static void updateUnstyledTexture(); // Update OpenGL unstyled frame texture
     };
 
 /**
@@ -1670,12 +1952,12 @@ double WindowQuad::originalBottom() const
 ***************************************************************/
 
 template <typename T>
-Motion<T>::Motion( T initial, double strength, double decay )
+Motion<T>::Motion( T initial, double strength, double smoothness )
     :   m_value( initial )
     ,   m_target( initial )
     ,   m_velocity()
     ,   m_strength( strength )
-    ,   m_decay( decay )
+    ,   m_smoothness( smoothness )
     {
     }
 
@@ -1685,7 +1967,7 @@ Motion<T>::Motion( const Motion &other )
     ,   m_target( other.target() )
     ,   m_velocity( other.velocity() )
     ,   m_strength( other.strength() )
-    ,   m_decay( other.decay() )
+    ,   m_smoothness( other.smoothness() )
     {
     }
 
@@ -1700,12 +1982,15 @@ void Motion<T>::calculate( const int msec )
     if( m_value == m_target && m_velocity == T() ) // At target and not moving
         return;
 
-    double delta = qMin( 1.0, double( msec ) / 100.0 );
-    T diff = m_target - m_value;
-    T strength = diff * m_strength;
-    m_velocity = m_decay * m_velocity * ( 1.0 - delta ) * ( 1.0 - delta )
-               + strength * delta; // TODO/HACK: Need to work out correct formula
-    m_value += m_velocity;
+    // Poor man's time independent calculation
+    int steps = qMax( 1, msec / 5 );
+    for( int i = 0; i < steps; i++ )
+        {
+        T diff = m_target - m_value;
+        T strength = diff * m_strength;
+        m_velocity = ( m_smoothness * m_velocity + strength ) / ( m_smoothness + 1.0 );
+        m_value += m_velocity;
+        }
     }
 
 template <typename T>
